(in-package "KPML")



(defun Express-Hearer-Q-Code (hearer onus)
  "Depends on the speech-act type of ONUS."
  (declare (ignore hearer))
  (if (or (term-type-p (fetch-feature 'speech-act onus) 'question)
	  (term-type-p (fetch-feature 'speech-act-id onus) 'question))
      'expresshearer
      'withholdhearer))


(defun Class-Q-Code (range)
  "If the argument is NOT a QUALITY, then return CLASS."
  (cond ((term-type-p range 'quality)
	 'nonclass)
	((and (term-type-p range 'um-set)
	      (loop for x in (term-graph-symbol (symbol-value range))
		  always (equal (class-q-code x) 'nonclass)))
	 'nonclass)
	(t 
	 'class)))

(defun Property-Q-Code (onus)
  "Is it a quality?"
  (cond ((numberp onus)
	 'notproperty)
	((or (term-type-p onus 'quality)
	     (and (term-type-p onus 'property-ascription)
		  (fetch-subc-feature 'domain onus)
		  (not (fetch-subc-feature 'range onus)))
	     (when (term-type-p (term-graph-parent (symbol-value onus)) 'object)
	       (and (term-type-p onus 'process)
		    (fetch-subc-feature 'participant onus))))
	 'property)
	((and (consp (symbol-value onus))
	      (every #'(lambda (x)
			 (equal (property-q-code X)
				'PROPERTY))
		     (symbol-value onus)))
	 'property)
	(t
	 'notproperty)))

;;; Also need to extend the following so that a set of qualities
;;; can be checked for scalability...

(defun Scalability-Q-Code (dimension)
  "If the DIMENSION's range is a SCALABLE-QUALITY, then SCALABLE."
  (let ((quality (fetch-subc-feature 'range dimension)))
    (if quality
	(cond ((and (term-type-p quality 'um-set)
		    (loop for p 
			in (term-graph-symbol (symbol-value quality))
			always (term-type-p p 'scalable-quality)))
	       'scalable)
	      ((term-type-p quality 'scalable-quality)
	       'scalable)
	      (t 'nonscalable))
      'nonscalable)))

;;; Should a set allow minimal attention?

(defun Minimal-Attention-Q-Code (obj)
  "If OBJ has no features, then minimal."
  (if (and (symbolp obj)
	   (boundp obj)
	   (typep (symbol-value obj) 'term-graph)
	   (null (term-graph-features (symbol-value obj)))
	   (not (term-type-p obj 'um-set)))
      'minimal
      'nonminimal))

;;; Following was not exhaustive when it should have been, was also
;;; incorrect due to not having been multilingualized properly...



(defun COMPARISON-MODIFICATION-Q-CODE (quality dimension)
  (declare (ignore dimension))
  (if (fetch-relation 'intensify-adjective-q quality) 
      'modification))

;;; Exceed-q extended... was previously just assuming it had to compare.
;;; Actually needs to see if the two is in the standard relationship to
;;; the one, and if so, whether the one is a greater-than-comparison.

(defun Exceed-Q-Code (one two)
  "Depends on the kind of parent relation."
  (if  (or
	(and 
	 (fetch-subc-feature 'standard one)
	 (equal (term-graph-symbol 
		 (symbol-value (fetch-subc-feature 'standard one)))
		(term-graph-symbol (symbol-value two)))
	 (eq (term-graph-type (symbol-value one)) 'greater-than-comparison))
	(and 
	 (fetch-subc-feature 'standard two)
	 (equal (term-graph-symbol 
		 (symbol-value (fetch-subc-feature 'standard two)))
		(term-graph-symbol (symbol-value one)))
	 (eq (term-graph-type (symbol-value two)) 'less-than-comparison))
	 )
      'exceed
    'nonexceed))


(defun Thousand-Unit-Q-Code (number)
  "Only if greater than 1000." 
  (if (>= number 1000)  'thousand 'notthousand))


(defun Unit-Value-ID-Code (number)
  "This is the basis of the Western number system (English), changing
   the base list changes the units the number system works on."
  (cond ((< number 20)
	 number)
	((>= number 1000000)
	 (floor (/ number 1000000)))
	((>= number 1000)
	 (floor (/ number 1000)))
	((>= number 100)
	 (floor (/ number 100)))
	((>= number 10)
	 number)
	(t number)))

(defun Cardinal-Complexity-Q-Code (number)
  "If number is less than 30, or a multiple of ten, it's expressed by a single position."
  (if (or (< number 30)
	  (and (< number 100)
	       (eql 0 (rem number 10)))
	  (and (not (< number 100)) 
	       (< number 1000)
	       (eql 0 (rem number 100)))
	  (and (not (<= number 1000))  ;; changed for 1000 
	       (< number 10000)
	       (eql 0 (rem number 1000)))
	  (and (not (< number 10000))
	       (> number 999999)
	       (eql 0 (rem number 1000000))))
      'singleposition
      'multiplepositions))

(defun Multiplicity-Q-Code (thing)
  "Look for number.  If THING isn't a plan, look for a plan for it.
   Even if it isn't a plan, look at other TERM-EQ-P plans for multiplicity
   (this is important for agreement in number in dependent clauses)."
  (if (and (symbolp thing)
	   (boundp thing)
	   (typep (symbol-value thing) 'term-graph))
      (if (or (eq (fetch-feature-symbol 'number thing) 'multiple)
	      (and (fetch-subc-feature 'quantity-ascription thing)
		   (let ((n (fetch-subc-feature 'quantity-ascription thing)))
		     (if (numberp n)
			 (< 1 n)
		       (and (term-type-p n 'number)              ;; 1.1 R3b11
			    (numberp (fetch-feature 'value n))
			    (< 1 (fetch-feature 'value n))))))
	      (term-type-p thing 'um-set)
	      (dolist (graph *plan-graphs*)
		(if (and (not (eq thing graph))
			 (term-eq-p thing graph)
			 (or (eq (fetch-feature 'multiplicity-q graph) 'multiple)
			     (eq (fetch-feature-symbol 'number graph) 'multiple)
			     (term-type-p graph 'um-set)))
		    (return t))))
	  'multiple)
      (if (null thing)
	  nil
	  (Multiplicity-Q-Code
	    (dolist (graph *plan-graphs*)
	      (if (term-eq-p graph thing)
		  (return)))))))

;;; new inquiry...
(defun NUMBER-VALUE-ID-CODE (spec)
  (if (and (symbolp spec)
	   (boundp spec)
	   (typep (symbol-value spec) 'term-graph)
	   (term-type-p spec 'number))
      (fetch-feature 'value spec)
    spec))

(defun NUMBER-RELATIVITY-Q-CODE (number)
  (if (or (numberp number)
	  (and (symbolp number)
	       (boundp number)
	       (typep (symbol-value number) 'term-graph)
	       (term-type-p number 'number)))
      'absolute 'relative))

